import os, json, tempfile, shutil
from pathlib import Path
import shutil

abstra_json = {"version": "0.2"}


def rm_tree(pth: Path):
    pth = Path(pth)
    if not pth.exists():
        return
    for child in pth.glob("*"):
        if child.is_file():
            child.unlink()
        else:
            rm_tree(child)
    pth.rmdir()


def init_dir():
    path = Path(tempfile.mkdtemp())
    abstra_json_path = path / "abstra.json"
    path.mkdir(exist_ok=True)
    if not abstra_json_path.exists():
        abstra_json_path.write_text(json.dumps(abstra_json))
    os.chdir(path)
    return path


def clear_dir(path: Path):
    os.chdir(path.parent)
    rm_tree(path)


def copy_dir(src, dst, symlinks=False, ignore=None):
    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            shutil.copytree(s, d, symlinks, ignore)
        else:
            shutil.copy2(s, d)



import typing
from uuid import uuid4 as uuid
from abstra.widgets.widget_base import Input


class Reactive(Input):
    type = "reactive"

    def __init__(self, callback: typing.Callable, **kwargs):
        self.set_props(dict(callback=callback, **kwargs))

    def has_errors(self):
        return False

    def render(self, context: dict):
        self.page = self.callback(context)
        if self.page is None:
            return []

        if self.value:
            self.page.set_values(self.value)
        return self.page.render(context)

    def serialize_value(self):
        return self.page.serialize_value() if self.page else []

    def set_props(self, props):
        self.callback = props.get("callback")
        self.set_value(props.get("initial_value", {}))
        self.page = None

    def validate(self):
        errors = []
        if self.page:
            errors.extend(self.page.validate())
        return errors

    def set_value(self, value):
        return super().set_value(value)

    def parse_value(self, value):
        if self.page:
            parsed = self.page.parse_value(value)
            return parsed
        else:
            return {}



import unittest
from pathlib import Path
from .utils import assert_form
import json
import os


class TestFormExamples(unittest.TestCase):
    def setUp(self) -> None:
        os.chdir(Path(__file__).parent)

    def test_back_and_forth(self):
        code_path = "./resources/test_run_steps/test_back_and_forth/code.py"
        msgs_path = "./resources/test_run_steps/test_back_and_forth/messages.json"
        code = open(code_path).read()
        msgs = json.load(open(msgs_path))
        assert_form(self, code, msgs, session_id="session-id")

    # def test_list_reactive(self):
    #     code_path = "./resources/test_run_steps/test_list_reactive/code.py"
    #     msgs_path = "./resources/test_run_steps/test_list_reactive/messages.json"
    #     code = open(code_path).read()
    #     msgs = json.load(open(msgs_path))
    #     assert_form(self, code, msgs, session_id="session-id")



import unittest
from abstra.widgets.library import ListInput
from abstra.forms.list_item_schema import ListItemSchema
from abstra.forms.page import Page

default_text_input = {
    "type": "text-input",
    "key": "foo",
    "label": "foo",
    "value": "",
    "placeholder": "Your value here",
    "required": True,
    "hint": None,
    "fullWidth": False,
    "mask": None,
    "disabled": False,
    "errors": [],
}


class TestListInput(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None

    def test_empty_case(self):
        schema = ListItemSchema().read("foo")
        input = ListInput("key", schema)

        rendered = input.render({})

        self.assertEqual(rendered["value"], [])

    def test_simple_case(self):
        schema = ListItemSchema().read("foo")
        input = ListInput("key", schema)

        self.assertEqual(input.render({})["value"], [])

        input.set_value([{"foo": "bar"}])

        self.assertEqual(
            input.serialize_value(),
            [{"foo": "bar"}],
        )

    def test_list_with_reactive(self):
        schema = ListItemSchema().reactive(lambda _: Page().read("foo"))
        list_input = ListInput("key", schema, initial_value=[dict(foo="bar")])
        self.assertEqual(list_input.value, [dict(foo="bar")])
        list_item_page = list_input.schemas[0]
        reactive_widget = list_item_page.widgets[0]
        self.assertEqual(
            reactive_widget.render({}), [{**default_text_input, "value": "bar"}]
        )
        self.assertEqual(
            list_input.value,
            [dict(foo="bar")],
        )

    def test_add_item(self):
        schema = ListItemSchema().reactive(lambda _: Page().read("foo"))

        page = Page().reactive(lambda _: Page().read_list(schema, key="list"))

        page.set_values(dict(list=[dict(list=[None])]))

        # page -> reactive -> list -> schema
        self.assertEqual(
            schema.render({}),
            [default_text_input],
        )

        self.assertEqual(
            page.widgets[0].value,
            dict(list=[dict(list=[None])]),
        )

        self.assertEqual(
            page.render({}),
            [
                {
                    "type": "list-input",
                    "key": "list",
                    "hint": None,
                    "errors": [],
                    "min": 0,
                    "max": None,
                    "addButtonText": "+",
                    "fullWidth": False,
                    "required": True,
                    "disabled": False,
                    "value": [{"foo": ""}],
                    "schemas": [
                        [
                            {
                                "type": "text-input",
                                "key": "foo",
                                "label": "foo",
                                "value": "",
                                "placeholder": "Your value here",
                                "required": True,
                                "hint": None,
                                "fullWidth": False,
                                "mask": None,
                                "disabled": False,
                                "errors": [],
                            }
                        ]
                    ],
                }
            ],
        )

    def test_list_reactive_dropdown(self):
        schema = ListItemSchema().reactive(
            lambda _: Page().read_dropdown("foo", ["bar", "baz"])
        )
        page = Page().read_list(schema, key="list")

        page.set_values(dict(list=[dict(foo=["bar"])]))

        self.assertEqual(
            page.render({}),
            [
                {
                    "type": "list-input",
                    "key": "list",
                    "hint": None,
                    "errors": [],
                    "min": 0,
                    "max": None,
                    "addButtonText": "+",
                    "fullWidth": False,
                    "required": True,
                    "disabled": False,
                    "value": [{"foo": ["bar"]}],
                    "schemas": [
                        [
                            {
                                "type": "dropdown-input",
                                "key": "foo",
                                "label": "foo",
                                "options": ["bar", "baz"],
                                "hint": None,
                                "multiple": False,
                                "placeholder": "Select your preference",
                                "value": ["bar"],
                                "required": True,
                                "fullWidth": False,
                                "disabled": False,
                                "errors": [],
                            }
                        ]
                    ],
                }
            ],
        )
